{- Copyright 2016 Joey Hess <id@joeyh.name>
 -
 - Licensed under the GNU AGPL version 3 or higher.
 -}

{-# LANGUAGE GeneralizedNewtypeDeriving #-}

module Types.Storage where

import Types

-- | All known locations where shares can be stored, ordered with
-- preferred locations first.
newtype StorageLocations = StorageLocations [Storage]
	deriving (Monoid)

-- | Storage interface. This can be used both for local storage,
-- an upload queue, or a remote server.
--
-- Note that there is no interface to enumerate shares.
-- This is intentional; servers should not allow that.
data Storage = Storage
	{ storeShare :: StorableObjectIdent -> Share -> IO StoreResult
	, retrieveShare :: ShareNum -> StorableObjectIdent -> IO RetrieveResult
	, obscureShares :: IO ObscureResult
	-- ^ Run after making some calls to storeShare/retrieveShare,
	-- to avoid correlation attacks.
	, countShares :: IO CountResult
	, moveShares :: Storage -> IO ()
	-- ^ Tries to move all shares from this storage to another one.
	}

data StoreResult = StoreSuccess | StoreAlreadyExists | StoreFailure String
	deriving (Show)

data RetrieveResult = RetrieveSuccess Share | RetrieveFailure String

data ObscureResult = ObscureSuccess | ObscureFailure String
	deriving (Show)

data CountResult = CountResult Integer | CountFailure String
	deriving (Show)
